package org.topbraid.spin.model;

import java.util.List;


/**
 * Base interface of the various SPARQL query types such as
 * Ask, Construct, Describe and Select.
 * 
 * @author Holger Knublauch
 */
public interface Query extends Command {
	
	/**
	 * Gets the list of URIs specified in FROM clauses.
	 * @return a List of URI Strings
	 */
	List<String> getFrom();
	
	
	/**
	 * Gets the list of URIs specified in FROM NAMED clauses.
	 * @return a List of URI Strings
	 */
	List<String> getFromNamed();
	
	
	/**
	 * Gets the ElementList of the WHERE clause of this query.
	 * Might be null or RDF.nil.
	 * @return the WHERE clause as an ElementList
	 */
	ElementList getWhere();

	
	/**
	 * Gets the elements in the WHERE clause of this query.
	 * The Elements will be typecast into the best suitable subclass.
	 * @return a List of Elements
	 */
	List<Element> getWhereElements();
}
