package org.topbraid.spin.model.impl;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.topbraid.spin.model.Select;
import org.topbraid.spin.model.Variable;
import org.topbraid.spin.model.print.PrintContext;
import org.topbraid.spin.vocabulary.SP;

import com.hp.hpl.jena.enhanced.EnhGraph;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.rdf.model.RDFList;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import com.hp.hpl.jena.vocabulary.RDF;


public class SelectImpl extends QueryImpl implements Select {

	public SelectImpl(Node node, EnhGraph eh) {
		super(node, eh);
	}

	
	public List<Variable> getResultVariables() {
		List<Variable> results = new LinkedList<Variable>();
		for(RDFNode node : getList(SP.resultVariables)) {
			results.add((Variable)node.as(Variable.class));
		}
		return results;
	}
	
	
	public boolean isDistinct() {
		return hasProperty(SP.distinct, getModel().createTypedLiteral(true));
	}
	
	
	public boolean isReduced() {
		return hasProperty(SP.reduced, getModel().createTypedLiteral(true));
	}


	public void print(PrintContext p) {
		printComment(p);
		printPrefixes(p);
		p.printIndentation(p.getIndentation());
		p.printKeyword("SELECT");
		p.print(" ");
		if(isDistinct()) {
			p.printKeyword("DISTINCT");
			p.print(" ");
		}
		if(isReduced()) {
			p.printKeyword("REDUCED");
			p.print(" ");
		}
		List<Variable> vars = getResultVariables();
		if(vars.isEmpty()) {
			p.print("*");
		}
		else {
			for(Iterator<Variable> vit = vars.iterator(); vit.hasNext(); ) {
				Variable var = vit.next();
				if(var.hasProperty(RDF.type, SP.Count) || var.hasProperty(RDF.type, SP.Sum)) {
					boolean as = false;
					String varName = "*";
					Statement varS = var.getProperty(SP.expression);
					if(varS != null && varS.getObject().isResource()) {
						Variable nestedVar = (Variable) varS.getResource().as(Variable.class);
						varName = nestedVar.toString();
					}
					
					Statement asS = var.getProperty(SP.as);
					if(asS != null && asS.getObject().isResource()) {
						as = true;
						p.print("(");
					}
					if(var.hasProperty(RDF.type, SP.Count)) {
						p.printKeyword("COUNT");
					}
					else {
						p.printKeyword("SUM");
					}
					p.print("(");
					
					if(var.hasProperty(SP.distinct, ResourceFactory.createTypedLiteral(true))) {
						p.print("DISTINCT ");
					}
					
					p.print(varName);
					if(as) {
						p.print(") ");
						p.printKeyword("AS");
						p.print(" ");
						Variable asVar = (Variable) asS.getResource().as(Variable.class);
						p.print(asVar.toString());
					}
					p.print(")");
				}
				else {
					var.print(p);
					// context.print(var.toString());
				}
				if(vit.hasNext()) {
					p.print(" ");
				}
			}
		}
		printStringFrom(p);
		p.println();
		printWhere(p);
		printGroupBy(p);
		printSolutionModifiers(p);
	}
	
	
	private void printGroupBy(PrintContext p) {
		Statement groupByS = getProperty(SP.groupBy);
		if(groupByS != null) {
			RDFList list = (RDFList) groupByS.getObject().as(RDFList.class);
			ExtendedIterator<RDFNode> it = list.iterator();
			if(it.hasNext()) {
				p.printIndentation(p.getIndentation());
				p.print(" ");
				p.printKeyword("GROUP BY");
				while(it.hasNext()) {
					p.print(" ");
					RDFNode node = it.next();
					printNestedExpressionString(p, node);
				}
			}
		}
	}
}
